searchData={"items":[{"type":"task","title":"mix check","doc":"One task to efficiently run all code analysis & testing tools in an Elixir project.","ref":"Mix.Tasks.Check.html"},{"type":"task","title":"Tools - mix check","doc":"Task comes out of the box with a rich predefined set of curated tools that are considered to be\nreasonable additions for most Elixir and Phoenix projects which care about having bug-free,\nmaintainable and secure code.\n\nFollowing standard library tools are configured by default:\n\n- [`:compiler`] - produces compilation warnings that allow to early detect bugs & typos in the\n  code eg. an attempt to call non-existing or deprecated function\n\n- [`:unused_deps`] - ensures that there are no unused dependencies in the project's `mix.lock`\n  file (e.g. after removing a previously used dependency)\n\n- [`:formatter`] - ensures that all the code follows the same basic formatting rules such as\n  maximum number of chars in a line or function indentation\n\n- [`:ex_unit`] - starts the application in test mode and runs all runtime tests against it\n  (defined as test modules or embedded in docs as doctests)\n\nFollowing community tools are configured by default:\n\n- [`:credo`] - ensures that all the code follows a further established set of software design,\n  consistency, readability & misc rules and conventions (still statically)\n\n- [`:dialyzer`] - performs static code analysis around type mismatches and other issues that are\n  commonly detected by static language compilers\n\n- [`:doctor`] - ensures that the project documentation is healthy by validating the presence of\n  module docs, functions docs, typespecs and struct typespecs\n\n- [`:ex_doc`] - compiles the project documentation in order to ensure that there are no issues\n  that would make it impossible for docs to get collected and assembled\n\n- [`:gettext`] - verifies that your POT files are up to date with the current state of the codebase\n\n- [`:mix_audit`] - scans the project's Mix dependencies for known Elixir security vulnerabilities\n  based on a GitHub-sourced list of security advisories\n\n- [`:npm_test`] - runs JavaScript tests in projects with front-end assets embedded in `assets`\n  directory and `package.json` in it (default for Phoenix apps)\n\n- [`:sobelow`] - performs security-focused static analysis mainly focused on the Phoenix\n  framework, but also detecting vulnerable dependencies in arbitrary Mix projects\n\nYou can disable or adjust curated tools as well as add custom ones via the configuration file.","ref":"Mix.Tasks.Check.html#module-tools"},{"type":"task","title":"Workflow - mix check","doc":"1. `:compiler` tool is run before others in order to compile the project just once and to avoid\n   reprinting the compilation error multiple times.\n\n2. If the compilation succeeded (even if with warnings), further tools are run in parallel while\n   their output is streamed live one by one for instant insight.\n\n3. Output from tools that have failed gets reprinted for sake of easily reading into them all at\n   once and identifying all project issues in one go.\n\n4. Summary is presented with a list of all tools that have failed, succeeded or were skipped due\n   to missing files or project dependencies.\n\n5. Manifest is written to specified file or tmp directory in order to allow running only failed\n   checks and for sake of reporting to CI.\n\n6. If any of the tools have failed, the Erlang system gets requested to emit exit status 1 upon\n   shutdown in order to make the CI build fail.\n\n#","ref":"Mix.Tasks.Check.html#module-workflow"},{"type":"task","title":"Tool order - mix check","doc":"Tools are run in parallel, but their output is presented one by one in order to avoid mixing it\nup. You can control the order in which the output is presented for tools that have started at the\nsame time via the `:order` tool option. You'll probably want to put tools that run quicker and\nfail more often before the others in order to get useful feedback as soon as possible. Curated\ntools are ordered in such a way out of the box.\n\n#","ref":"Mix.Tasks.Check.html#module-tool-order"},{"type":"task","title":"Tool processes and ANSI formatting - mix check","doc":"Tools are run in separate processes. This has following benefits:\n\n- allows to run tools in parallel & stream their output\n- catches exit statuses in order to detect failures\n- enables running Mix tasks in multiple envs\n- enables including non-Elixir scripts and tools in the check\n\nThe downside is that tools will be run outside of TTY which will usually result in disabling ANSI\nformatting. This issue is fixed in different ways depending on Elixir version:\n\n- **Elixir 1.9 and newer**: patches all Elixir commands and Mix tasks with `--erl-config` option\n  to load the Erlang configuration provided by `ex_check` that sets the `ansi_enabled` flag\n\n- **older versions**: patches Mix tasks with `--eval` option to run `Application.put_env/3` that\n  sets the `ansi_enabled` flag\n\nYou may keep your Elixir commands unaffected via the `:enable_ansi` tool option. It's ignored for\nnon-Elixir tools for which you'll have to enforce ANSI on your own.\n\n#","ref":"Mix.Tasks.Check.html#module-tool-processes-and-ansi-formatting"},{"type":"task","title":"Cross-tool dependencies - mix check","doc":"Even though tools are run in parallel, it's possible to make sure that specific tool will be run\nonly after other(s) are completed via the `:deps` tool option. This enables defining complex\nworkflows, such as the following:\n\n- tools may reuse artifacts from ones executed earlier\n- tools may handle the success/failure of those they depend on\n- tools may be forced not to run at the same time without giving up on entire parallel execution\n\nBy default tools will be run regardless of the exit status of their dependencies, but it's\npossible to depend on specific exit status via the `:status` dependency option. Tools will not be\nrun if their dependencies won't get to run at all e.g. due to using `--except` command line option\nor a missing/circular dependency.\n\n#","ref":"Mix.Tasks.Check.html#module-cross-tool-dependencies"},{"type":"task","title":"Umbrella projects - mix check","doc":"Task comes with extensive support for umbrella projects. The most notable feature is the ability\nto run tools recursively for each child app separately. It's similar to flagging Mix tasks as\nrecursive but empowered with following extra benefits:\n\n- runs recursively not just Mix tasks, but also arbitrary scripts & commands\n- runs tools on child apps in parallel\n- allows tools to target only specific child apps\n- presents failures & run durations for each child app separately\n- detects if curated tools should run for each child app separately\n- builds separate cross-tool dependency chains for each child app\n\nYou may want to disable parallel execution of the tool on child apps (`parallel: false` under\n`:umbrella` tool option) if it uses the same resources across tool runs against different child\napps. An example of that could be `ex_unit` that, depending on a project and test dependencies,\nmay involve mutating the same database in test suites belonging to separate child apps.\n\nYou may have the tool run *only* at the root level of the umbrella by disabling the recursive\nexecution (`recursive: false` under `:umbrella` tool option) and targeting an empty list of child\napps (`apps: []` under `:umbrella` tool option).\n\n#","ref":"Mix.Tasks.Check.html#module-umbrella-projects"},{"type":"task","title":"Retrying failed tools - mix check","doc":"You may run only failed tools in the next run by passing the `--retry` command line option in\norder to avoid wasting time on checks that have already passed.\n\nIn addition, some tools offer the capability to do the same, i.e. run only failed tests or checks.\nIf the tool provides such capability, it will be automatically executed this way when the\n`--retry` command line option is passed. This feature is provided out of the box for `ex_unit`\ntool and may be provided for any tool via `:retry` tool option in config.\n\nTask will run in retry mode automatically even if `--retry` was not specified when a previous run\nhas resulted in any failures. You can change this behavior with `--no-retry` command line option\nor by setting `retry: false` in config.\n\n#","ref":"Mix.Tasks.Check.html#module-retrying-failed-tools"},{"type":"task","title":"Fix mode - mix check","doc":"Some tools are capable of automatically resolving issues by running in the fix mode. You may take\nadvantage of this feature by passing the `--fix` command line option. This feature is provided out\nof the box for `formatter` and `unused_deps` tools and may be provided for any tool via `:fix`\ntool option in config.\n\nYou may combine `--fix` with `--retry` to only request tools that have failed to do the fixing.\n\nYou may also consider adding `~/.check.exs` with `[fix: true]` on a local machine in order to\nalways run in the fix mode for convenience. You probably don't want this option in the project\nconfig as that would trigger fix mode on CI as well - unless you want CI to perform & commit back\nfixes.\n\n#","ref":"Mix.Tasks.Check.html#module-fix-mode"},{"type":"task","title":"Manifest file - mix check","doc":"After every run, task writes a list of tool statuses to manifest file specified with `--manifest`\ncommand line option or to temp directory. This allows to run only failed tools in the next run by\npassing the `--retry` command line option, but manifest file may also be used for sake of\nreporting to CI.\n\nIt's a simple plain text file with following syntax that should play well with shell commands:\n\n    PASS compiler\n    FAIL formatter\n    PASS ex_unit\n    PASS unused_deps\n    SKIP credo\n    SKIP sobelow\n    SKIP ex_doc\n    SKIP dialyzer\n    SKIP mix_audit","ref":"Mix.Tasks.Check.html#module-manifest-file"},{"type":"task","title":"Configuration file - mix check","doc":"Check configuration may be adjusted with the optional `.check.exs` file.\n\nConfiguration file should evaluate to keyword list with following options:\n\n- `:parallel` - toggles running tools in parallel; default: `true`\n- `:skipped` - toggles printing skipped tools in summary; default: `true`\n- `:fix` - toggles running tools in fix mode in order to resolve issues automatically; default: `false`\n- `:retry` - toggles running only checks that have failed in the last run; default: 'true' if manifest exists\n- `:tools` - a list of tools to run; default: curated tools; more info below\n\nTool list under `:tools` key may contain following tool tuples:\n\n- `{:tool_name, opts}`\n- `{:tool_name, enabled}` where `enabled` corresponds to the `:enabled` option\n- `{:tool_name, command}` where `command` corresponds to the `:command` option\n- `{:tool_name, command, opts}` where `command` corresponds to the `:command` option\n\nTool options (`opts` above) is a keyword list with following options:\n\n- `:enabled` - enables/disables already defined tools; default: `true`\n- `:command` - command as string or list of strings (executable + arguments)\n- `:cd` - directory (relative to cwd) to change to before running the command\n- `:env` - environment variables as map with string keys & values\n- `:order` - integer that controls the order in which tool output is presented; default: `0`\n- `:deps` - list of tools that the given tool depends on; more info below\n- `:enable_ansi` - toggles extending Elixir/Mix commands to have ANSI enabled; default: `true`\n- `:umbrella` - configures the tool behaviour in an umbrella project; more info below\n- `:fix` - fix mode command as string or list of strings (executable + arguments)\n- `:retry` - command to retry after failure as string or list of strings (executable + arguments)\n\nDependency list under `:deps` key may contain `:tool_name` atoms or `{:tool_name, opts}` tuples\nwhere `opts` is a keyword list with following options:\n\n- `:status` - depends on specific exit status; one of `:ok`, `:error`, exit code integer or a list\n  with any of the above; default: any exit status\n- `:else` -  specifies the behaviour upon dependency being unsatisfied; one of `:skip` (show the\n  tool among skipped ones), `:disable` (disable the tool without notice); default: `:skip`\n\nUmbrella configuration under `:umbrella` key is a keyword list with following options:\n\n- `:recursive` - toggles running the tool on each child app separately as opposed to running it\n  once from umbrella root; default: `true` except for non-recursive Mix tasks\n- `:parallel` - toggles running tool in parallel on all child apps; default: `true`\n- `:apps` - list of umbrella child app names targeted by the tool; default: all apps\n\nTask will load the configuration in following order:\n\n1. Default stock configuration.\n2. `--config` file opt on command line.\n3. `.check.exs` in user home directory.\n4. `.check.exs` in current project directory (or umbrella root for an umbrella project).\n\nUse the `mix check.gen.config` task to generate sample configuration that comes with well-commented examples to help you get started.","ref":"Mix.Tasks.Check.html#module-configuration-file"},{"type":"task","title":"Command line options - mix check","doc":"- `--config path/to/check.exs` - override default config file\n- `--manifest path/to/manifest` - specify path to file that holds last run results\n- `--only dialyzer --only credo ...` - run only specified check(s)\n- `--except dialyzer --except credo ...` - don't run specified check(s)\n- `--[no-]fix` - (don't) run tools in fix mode in order to resolve issues automatically\n- `--[no-]retry` - (don't) run only checks that have failed in the last run\n- `--[no-]parallel` - (don't) run tools in parallel\n- `--[no-]skipped` - (don't) print skipped tools in summary\n\n[`:compiler`]: https://hexdocs.pm/mix/Mix.Tasks.Compile.html\n[`:credo`]: https://hexdocs.pm/credo\n[`:dialyzer`]: https://hexdocs.pm/dialyxir\n[`:doctor`]: https://github.com/akoutmos/doctor\n[`:ex_doc`]: https://hexdocs.pm/ex_doc\n[`:ex_unit`]: https://hexdocs.pm/ex_unit\n[`:gettext`]: https://hexdocs.pm/gettext\n[`:formatter`]: https://hexdocs.pm/mix/Mix.Tasks.Format.html\n[`:npm_test`]: https://docs.npmjs.com/cli/test.html\n[`:sobelow`]: https://hexdocs.pm/sobelow\n[`:unused_deps`]: https://hexdocs.pm/mix/Mix.Tasks.Deps.Unlock.html\n[`:mix_audit`]: https://hexdocs.pm/mix_audit","ref":"Mix.Tasks.Check.html#module-command-line-options"},{"type":"task","title":"mix check.gen.config","doc":"Generates optional configuration file (`.check.exs`) for adjusting the check.","ref":"Mix.Tasks.Check.Gen.Config.html"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/),\nand this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).\n\n## [Unreleased]\n\nNothing yet.\n\n## [0.16.0] - 2024-03-01\n\n- **Changed** minimum supported Elixir version from Elixir 1.7 to Elixir 1.12\n- **Added** `gettext` tool\n- **Fixed** incorrect relative path handling in Elixir 1.16\n\n## [0.15.0] - 2022-12-05\n\n- **Added** `mix_audit` tool\n- **Fixed** retry mode info to properly split tool names\n- **Improved** docs with numerous fixes & refinements\n\n## [0.14.0] - 2021-01-26\n\n- **Added** `--fix` command line option to resolve issues automatically for tools that provide the fix command via the `:fix` tool option\n- **Added** automatic toggling of retry mode when previous run resulted in failures (pass `--no-retry` to override)\n- **Added** `:doctor` tool\n- **Improved** retry mode with capability to run only failed tests or checks for tools that provide the retry command via the `:retry` tool option\n- **Improved** default tool ordering to persist when custom config is applied to default tools\n- **Renamed** `--failed` command line option to `--retry`\n\n## [0.13.0] - 2020-12-23\n\n- **Added** `:unused_deps` tool\n- **Added** `--config` command line option to point to arbitrary configuration file path\n- **Added** `--failed` command line option to only run checks that have failed in the last run\n- **Added** `--manifest` command line option to specify path to file that holds last run results\n\n## [0.12.0] - 2020-07-06\n\n- **Added** `:deps` tool option with support for depending on specific exit status\n- **Removed** `:run_after` tool option (please use `:deps` tool option instead)\n- **Fixed** merging tool umbrella opts with those set in ancestor config\n- **Updated** default dialyxir config to no longer include `--halt-exit-status` deprecated in\n  [1.0.0-rc.7](https://github.com/jeremyjh/dialyxir/blob/master/CHANGELOG.md#100-rc7---2019-09-21)\n- **Improved** test suite detection to only check for `test` directory instead of `test_helper.exs`\n\n## [0.11.0] - 2019-09-07\n\n- **Added** support for parallel & sequential recursive tool execution in umbrella projects\n- **Added** support for tool to target only specific umbrella child app(s)\n- **Added** `:npm_test` tool for seamless integration of testing assets in Phoenix projects\n- **Added** tool skipping if `cd` tool option points to non-existing directory\n- **Added** `{:tool_name, command, opts}` shorthand tool tuples\n- **Added** umbrella recursive flag to tool `sobelow` (fixing it for umbrella projects)\n- **Changed** tool autodetection to support detection order and disabling instead of skipping\n- **Fixed** `{:tool_name, command}` tool tuples to support lists of strings for commands\n- **Removed** `:require_files` and `:require_deps` tool configuration options\n- **Removed** `--skip` option from default configuration for tool `sobelow`\n\n## [0.10.0] - 2019-08-18\n\n- **Added** `:run_after` tool coonfiguration option (introducing powerful tool dependency system)\n\n## [0.9.0] - 2019-08-01\n\n- **Added** automatic ANSI enabling for arbitrary Elixir commands (and not just Mix tasks)\n- **Added** support for shorthand tool configuration (`{:tool_name, true/false/\"command\"}`)\n- **Fixed** starting of app in Mix tasks that should have it started (e.g. `mix test`)\n\n## [0.8.0] - 2019-07-31\n\n- **Changed** emit exit code via `System.at_exit/1` instead of `System.halt/1`\n- **Fixed** accidental starting of app in Mix tasks\n- **Removed** `--no-exit-status` command line option\n- **Removed** `:exit_status` configuration option\n\n## [0.7.0] - 2019-07-29\n\n- **Changed** automatic ANSI enabling for Mix tasks to use `mix run` instead of `mix check.run`\n- **Removed** `mix check.run` task\n\n## [0.6.0] - 2019-07-26\n\n- **Added** `:order` tool coonfiguration option\n- **Changed** check summary to sort the items by status and name\n- **Fixed** re-enabling tools after disable in ancestor config\n- **Fixed** merging env vars with those set in ancestor config\n- **Fixed** detection of Mix env for `mix check.run` wrapper\n\n## [0.5.0] - 2019-07-24\n\n- **Added** automatic ANSI enabling for Mix tasks by auto-prepending `mix check.run`\n- **Added** `:cd` tool coonfiguration option\n- **Added** `:env` tool coonfiguration option\n- **Added** support for invoking shell scripts as tools\n\n## [0.4.0] - 2019-07-23\n\nNo user-facing changes.\n\n## [0.3.0] - 2019-07-22\n\n- **Added** ANSI enabling for Mix tasks by prepending `mix check.run` in tool command\n\n## [0.2.0] - 2019-07-19\n\n- **Added** `sobelow` tool\n- **Added** loading of ancestor config (home + umbrella root)\n- **Changed** Elixir version requirement from `1.9` to `1.7`\n\n## [0.1.0] - 2019-07-15\n\nInitial release.\n\n[Unreleased]: https://github.com/karolsluszniak/ex_check/compare/v0.14.0...HEAD\n[0.14.0]: https://github.com/karolsluszniak/ex_check/compare/v0.13.0...v0.14.0\n[0.13.0]: https://github.com/karolsluszniak/ex_check/compare/v0.12.0...v0.13.0\n[0.12.0]: https://github.com/karolsluszniak/ex_check/compare/v0.11.0...v0.12.0\n[0.11.0]: https://github.com/karolsluszniak/ex_check/compare/v0.10.0...v0.11.0\n[0.10.0]: https://github.com/karolsluszniak/ex_check/compare/v0.9.0...v0.10.0\n[0.9.0]: https://github.com/karolsluszniak/ex_check/compare/v0.8.0...v0.9.0\n[0.8.0]: https://github.com/karolsluszniak/ex_check/compare/v0.7.0...v0.8.0\n[0.7.0]: https://github.com/karolsluszniak/ex_check/compare/v0.6.0...v0.7.0\n[0.6.0]: https://github.com/karolsluszniak/ex_check/compare/v0.5.0...v0.6.0\n[0.5.0]: https://github.com/karolsluszniak/ex_check/compare/v0.4.0...v0.5.0\n[0.4.0]: https://github.com/karolsluszniak/ex_check/compare/v0.3.0...v0.4.0\n[0.3.0]: https://github.com/karolsluszniak/ex_check/compare/v0.2.0...v0.3.0\n[0.2.0]: https://github.com/karolsluszniak/ex_check/compare/v0.1.0...v0.2.0\n[0.1.0]: https://github.com/karolsluszniak/ex_check/releases/tag/v0.1.0","ref":"changelog.html"},{"type":"extras","title":"License","doc":"# The MIT License\n\nCopyright (c) 2019 Karol Słuszniak\n\nPermission is hereby granted, free of charge, to any person obtaining\na copy of this software and associated documentation files (the\n\"Software\"), to deal in the Software without restriction, including\nwithout limitation the rights to use, copy, modify, merge, publish,\ndistribute, sublicense, and/or sell copies of the Software, and to\npermit persons to whom the Software is furnished to do so, subject to\nthe following conditions:\n\nThe above copyright notice and this permission notice shall be\nincluded in all copies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND,\nEXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF\nMERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND\nNONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE\nLIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION\nOF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION\nWITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.","ref":"license.html"},{"type":"extras","title":"Overview","doc":"# ![ex_check](./assets/logo-with-name.svg)\n\n[![Hex version](https://img.shields.io/hexpm/v/ex_check.svg?color=hsl(265,40%,60%))](https://hex.pm/packages/ex_check)\n[![Hex docs](https://img.shields.io/badge/hex-docs-lightgreen.svg?color=hsl(265,40%,60%))](https://hexdocs.pm/ex_check/)\n[![Build status](https://img.shields.io/github/actions/workflow/status/karolsluszniak/ex_check/check.yml?branch=master)](https://github.com/karolsluszniak/ex_check/actions)\n[![Downloads](https://img.shields.io/hexpm/dt/ex_check.svg)](https://hex.pm/packages/ex_check)\n[![License](https://img.shields.io/github/license/karolsluszniak/ex_check.svg)](https://github.com/karolsluszniak/ex_check/blob/master/LICENSE.md)\n[![Last updated](https://img.shields.io/github/last-commit/karolsluszniak/ex_check.svg)](https://github.com/karolsluszniak/ex_check/commits/master)\n\n![Demo](./assets/demo-67x16.svg)\n\n**Run all code checking tools with a single convenient `mix check` command.**\n\n---\n\nTakes seconds to setup, saves hours in the long term.\n- Comes out of the box with a [predefined set of curated tools](https://hexdocs.pm/ex_check/Mix.Tasks.Check.html#module-tools)\n- Delivers results faster by [running tools in parallel and catching all issues in one go](https://hexdocs.pm/ex_check/Mix.Tasks.Check.html#module-workflow)\n- Checks the project consistently on every developer's local machine & [on the CI](https://github.com/karolsluszniak/ex_check#continuous-integration)\n- Runs only the tools & tests that have [failed in the last run](https://hexdocs.pm/ex_check/Mix.Tasks.Check.html#module-retrying-failed-tools)\n- Fixes issues automatically in [the fix mode](https://hexdocs.pm/ex_check/Mix.Tasks.Check.html#module-fix-mode)\n\nSports powerful features to enable ultimate flexibility.\n- Add custom mix tasks, shell scripts and commands via [configuration file](https://hexdocs.pm/ex_check/Mix.Tasks.Check.html#module-configuration-file)\n- Enhance you CI workflow to [report status](https://hexdocs.pm/ex_check/Mix.Tasks.Check.html#module-manifest-file), [retry random failures](#random-failures) or [autofix issues](#autofixing)\n- Empower umbrella projects with [parallel recursion over child apps](https://hexdocs.pm/ex_check/Mix.Tasks.Check.html#module-umbrella-projects)\n- Design complex parallel workflows with [cross-tool deps](https://hexdocs.pm/ex_check/Mix.Tasks.Check.html#module-cross-tool-dependencies)\n\nTakes care of the little details, so you don't have to.\n- Compiles the project and collects compilation warnings in one go\n- Ensures that output from tools is [ANSI formatted & colorized](https://hexdocs.pm/ex_check/Mix.Tasks.Check.html#module-tool-processes-and-ansi-formatting)\n- Retries ExUnit with the `--failed` flag\n\nRead more in the introductory [\"One task to rule all Elixir analysis & testing tools\"](https://cloudless.studio/one-task-to-rule-all-elixir-analysis-testing-tools) article.","ref":"readme.html"},{"type":"extras","title":"Getting started - Overview","doc":"Add `ex_check` dependency in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:ex_check, \"~> 0.14.0\", only: [:dev], runtime: false}\n  ]\nend\n```\n\nFetch the dependency:\n\n```\nmix deps.get\n```\n\nRun the check:\n\n```\nmix check\n```\n\nThat's it - `mix check` will detect and run all the available tools.\n\n#","ref":"readme.html#getting-started"},{"type":"extras","title":"Community tools - Overview","doc":"If you want to take advantage of community curated tools, add following dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:credo, \">= 0.0.0\", only: [:dev], runtime: false},\n    {:dialyxir, \">= 0.0.0\", only: [:dev], runtime: false},\n    {:doctor, \">= 0.0.0\", only: [:dev], runtime: false},\n    {:ex_doc, \">= 0.0.0\", only: [:dev], runtime: false},\n    {:gettext, \">= 0.0.0\", only: [:dev], runtime: false},\n    {:sobelow, \">= 0.0.0\", only: [:dev], runtime: false},\n    {:mix_audit, \">= 0.0.0\", only: [:dev], runtime: false}\n  ]\nend\n```\n\nYou may also generate `.check.exs` to adjust the check:\n\n```\nmix check.gen.config\n```\n\nAmong others, this allows to permanently disable specific tools and avoid the skipped notices.\n\n```elixir\n[\n  tools: [\n    {:dialyzer, false},\n    {:sobelow, false}\n  ]\n]\n```\n\n#","ref":"readme.html#community-tools"},{"type":"extras","title":"Local-only fix mode - Overview","doc":"You should keep local and CI configuration as consistent as possible by putting together the project-specific `.check.exs`. Still, you may introduce local-only config by creating the `~/.check.exs` file. This may be useful to enforce global flags on all local runs. For example, the following config will enable the fix mode in local (writable) environment:\n\n```elixir\n[\n  fix: true\n]\n```\n\n> You may also [enable the fix mode on the CI](#autofixing).","ref":"readme.html#local-only-fix-mode"},{"type":"extras","title":"Documentation - Overview","doc":"Learn more about the tools included in the check as well as its workflow, configuration and options [on HexDocs](https://hexdocs.pm/ex_check/Mix.Tasks.Check.html) or by running `mix help check`.\n\nWant to write your own code check? Get yourself started by reading the [\"Writing your first Elixir code check\"](https://cloudless.studio/writing-your-first-elixir-code-check) article.","ref":"readme.html#documentation"},{"type":"extras","title":"Continuous Integration - Overview","doc":"With `mix check` you can consistently run the same set of checks locally and on the CI. CI configuration also becomes trivial and comes out of the box with parallelism and error output from all checks at once regardless of which ones have failed.\n\nLike on a local machine, all you have to do in order to use `ex_check` on CI is run `mix check` instead of `mix test`. This repo features working CI configs for following providers:\n\n- GitHub - [.github/workflows/check.yml](https://github.com/karolsluszniak/ex_check/blob/master/.github/workflows/check.yml)\n\nYes, `ex_check` uses itself on the CI. Yay for recursion!\n\n#","ref":"readme.html#continuous-integration"},{"type":"extras","title":"Autofixing - Overview","doc":"You may automatically fix and commit back trivial issues by triggering the fix mode on the CI as well. In order to do so, you'll need a CI script or workflow similar to the example below:\n\n```bash\nmix check --fix && \\\n  git diff-index --quiet HEAD -- && \\\n  git config --global user.name 'Autofix' && \\\n  git config --global user.email 'autofix@example.com' && \\\n  git add --all && \\\n  git commit --message \"Autofix\" && \\\n  git push\n```\n\nFirst, we perform the check in the fix mode. Then, if no unfixable issues have occurred and if fixes were actually made, we proceed to commit and push these fixes.\n\nOf course your CI will need to have write permissions to the source repository.\n\n#","ref":"readme.html#autofixing"},{"type":"extras","title":"Random failures - Overview","doc":"You may take advantage of the automatic retry feature to efficiently re-run failed tools & tests multiple times. For instance, following shell command runs check up to three times: `mix check || mix check || mix check`. And here goes an alternative without the logical operators:\n\n```bash\nmix check\nmix check --retry\nmix check --retry\n```\n\nThis will work as expected because the `--retry` flag will ensure that only failed tools are executed, resulting in no-op if previous run has succeeded.","ref":"readme.html#random-failures"},{"type":"extras","title":"Troubleshooting - Overview","doc":"#","ref":"readme.html#troubleshooting"},{"type":"extras","title":"Duplicate builds - Overview","doc":"If, as suggested above, you've added `ex_check` and curated tools to `only: [:dev]`, you're keeping the test environment reserved for `ex_unit`. While a clean setup, it comes at the expense of Mix having to compile your app twice - in order to prepare `:test` build just for `ex_unit` and `:dev` build for other tools. This costs precious time both on local machine and on the CI. It may also cause issues if you set `MIX_ENV=test`, which is a common practice on the CI.\n\nYou may avoid this issue by running `mix check` and all the tools it depends on in the test environment. In such case you may want to have the following config in `mix.exs`:\n\n```elixir\ndef project do\n  [\n    # ...\n    preferred_cli_env: [\n      check: :test,\n      credo: :test,\n      dialyzer: :test,\n      doctor: :test,\n      sobelow: :test,\n      \"deps.audit\": :test\n    ]\n  ]\nend\n\ndef deps do\n  [\n    {:credo, \">= 0.0.0\", only: [:test], runtime: false},\n    {:dialyxir, \">= 0.0.0\", only: [:test], runtime: false},\n    {:doctor, \">= 0.0.0\", only: [:test], runtime: false},\n    {:ex_check, \"~> 0.14.0\", only: [:test], runtime: false},\n    {:ex_doc, \">= 0.0.0\", only: [:dev, :test], runtime: false},\n    {:sobelow, \">= 0.0.0\", only: [:test], runtime: false},\n    {:mix_audit, \">= 0.0.0\", only: [:test], runtime: false}\n  ]\nend\n```\n\nAnd the following in `.check.exs`:\n\n```elixir\n[\n  tools: [\n    {:compiler, env: %{\"MIX_ENV\" => \"test\"}},\n    {:formatter, env: %{\"MIX_ENV\" => \"test\"}},\n    {:ex_doc, env: %{\"MIX_ENV\" => \"test\"}}\n  ]\n]\n```\n\nAbove setup will consistently check the project using just the test build, both locally and on the CI.\n\n### `unused_deps` false negatives\n\nYou may encounter an issue with the `unused_deps` check failing on the CI while passing locally, caused by fetching only dependencies for specific env. If that happens, remove the `--only test` (or similar) from your `mix deps.get` invocation on the CI to fix the issue.","ref":"readme.html#duplicate-builds"},{"type":"extras","title":"Changelog - Overview","doc":"See [CHANGELOG.md](./CHANGELOG.md).","ref":"readme.html#changelog"},{"type":"extras","title":"Copyright and License - Overview","doc":"Copyright (c) 2019 Karol Słuszniak\n\nThis work is free. You can redistribute it and/or modify it under the\nterms of the MIT License. See the [LICENSE.md](./LICENSE.md) file for more details.","ref":"readme.html#copyright-and-license"}],"content_type":"text/markdown"}